/*
PiAnoS (Picture Annotation System), A fingerprint examiner training tool
Copyright (C) 2011-2014 University of Lausanne, Switzerland
https://ips-labs.unil.ch/pianos

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as
published by the Free Software Foundation, either version 3 of the
License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see http://www.gnu.org/licenses/.
*/

#define FUSE_USE_VERSION 26

#include <unistd.h>
#include <sys/types.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <time.h>

#include <fuse.h>

time_t startup_time;


/* What the code should look like on the C++ side (low-level)
- try {} catch{} converts exceptions to errnos
- return values as pre-filled as much as possible
int Directory::getattr_base(stat*)
{
	try
	{
		st = getattr();
		ret->st_mode = S_IFDIR | (0777 & st.mode);
	}
	catch (Error& e)
	{
		return e.code;
	}
}
*/

/* What the code should look like on the C++ side (high-level)
- errors raise exceptions
Stat* Directory::getattr()
{
	s = Stat(0755);
	s.setAccessTime(new Date());
	return s;
}
*/

int fuse_hook__getattr(const char* path, struct stat* buf)
{
	//memset(buf, 0, sizeof(struct stat));
	//buf->st_size = 42;
	buf->st_uid = getuid();
	buf->st_gid = getgid();
	buf->st_mode = S_IFDIR | 0755;
	buf->st_atime = time(NULL);
	buf->st_ctime = startup_time;
	buf->st_mtime = startup_time;
	printf("%s\n", path);
	return 0;
}

#define FUSE_TABLE_START(varname) struct fuse_operations varname = {
#define FUSE_MAP(cb) . cb = fuse_hook__##cb
#define FUSE_TABLE_END() };


FUSE_TABLE_START(fuse_operations_binding)
	FUSE_MAP(getattr)
FUSE_TABLE_END()

int main(int argc, char *argv[])
{
    int ret;
	
	startup_time = time(NULL);
	
	/* Don't run as root for security reasons */
    if (getuid() == 0 || geteuid() == 0) 
	{
		fprintf(stderr, "Cannot run as root\n");
		return 1;
    }
    
    /* Call Fuse main loop */
    fprintf(stderr, "Entering fuse loop\n");
    ret = fuse_main(argc, argv, &fuse_operations_binding, NULL);
    fprintf(stderr, "Exiting fuse loop %d\n", ret);
    
    return ret;
}
